## Included tests

#### HighLevelTest
Reproduces race condition in ObjectInputStream when deserializing proxy calss
#### LowLevelTest
Reproduces race condition in ObjectInputStream.readProxyDesc method

In both cases test simultaneously starts multiple jvms, and multiple threads within each jvm to provoke race condition

It takes up to 30 minutes to reproduce this issue (usually much faster).
It is reproducible on Oracle JDK 1.7.0_60, 1.8.0_92 and OpenJDK 1.8.0_111
Test does not fail on Oracle JDK 1.6.0_45

Proposed fix may be found in java.lang.reflect.WeakCache - implementation is based on locking each Supplier with
it's own ReadWriteLock.

### Issue details:
The issue with **java.io.InvalidClassException: Not a proxy** has been spotted. It appears to be bug in JDK, which  relates to concurrency during jvm startup and rather than to garbage collection, as we supposed at the beginning.
There is an open Oracle JDK bug, which lacks test to reproduce the issue to be fixed: http://bugs.java.com/bugdatabase/view_bug.do?bug_id=8087168

Here is what happens(java source code version "1.8.0_45"):
During application service startup we trigger multiple rmi calls to configured jvms to initialize active subscriptions – new thread in spawned for each rmi call.
So, almost simultaneously multiple threads execute the following code concurrently:

**java.io.ObjectInputStream**:

```java
1555:Class cl = null;
1559:if ((cl = resolveProxyClass(ifaces)) == null) {
1560:    throw new ClassNotFoundException("null class");
1561:} else if (!Proxy.isProxyClass(cl)) {
1562:    throw new InvalidClassException("Not a proxy");
1563:}
```

Where ifaces is an array of Strings, representing interfaces, for which we would like to generate dynamic proxy(see java.io.ObjectInputStream#resolveProxyClass javadoc).
Under the hood **resolveProxyClass** method executes the following code:

**java.lang.reflect.Proxy**:

```java
419: return proxyClassCache.get(loader, interfaces);
```

where proxyClassCache is a statically-referenced WeakCache instance:

**java.lang.reflect.Proxy**:

```java
239:private static final WeakCache<ClassLoader, Class<?>[], Class<?>>
240:    proxyClassCache = new WeakCache<>(new KeyFactory(), new ProxyClassFactory());
```

proxyClassCache.get creates new instance of proxy class using ProxyClassFactory passed in constructor, if such instance does not exist.

Now, isProxyClass method uses the following code:

**java.lang.reflect.Proxy**:

```java
790:public static boolean isProxyClass(Class<?> cl) {
791:    return Proxy.class.isAssignableFrom(cl) && proxyClassCache.containsValue(cl);
792:}
```

So we get dynamic proxy from proxyClassCache.get, but when proxyClassCache.containsValue is executed - it returns false.
How is it possible? Dynamic proxy class should be in proxyClassCache already to that moment.
The answer is the following(all code from java.lang.reflect.WeakCache):

Thread1 ->
* comes to line 121: Supplier<V> supplier = valuesMap.get(subKey); (subKey generated based on passed interfaces; valuesMap contains mapping subKey -> supplier)
* supplier is null, so go to line 138: factory = new Factory(key, parameter, subKey, valuesMap)
* goes to line 145: supplier = factory
* goes to line 127, as supplier is not null: V value = supplier.get()
* goes to synchronized Factory#get
* creates new CacheValue and associates it with current subKey:

```java
240:CacheValue<V> cacheValue = new CacheValue<>(value);
243:if (valuesMap.replace(subKey, this, cacheValue)) {
251: reverseMap.put(cacheValue, Boolean.TRUE);
```

* after execution of line 243 processor switches to the next thread, so line 251 is not executed
Thread2 ->
* comes to line 121 : Supplier<V> supplier = valuesMap.get(subKey); and retrieves cache value, which was stored at line 243 by Thread1 ( line 121 is outside synchronized Factory#get)
* goes to line 127 – and as CacheValue#get is not synchronized, so Thread2 successfully retrieves proxy without lock.
* goes to isProxyClass method and tries to perform the following check:

```java
179: return reverseMap.containsKey(new LookupValue<>(value));
```

* as Thread1 had no chance to put value in reverseMap - containsKey returns false in Thread2, which results in

**java.io.InvalidClassException: Not a proxy**

So – this bug is a result of poor synchronization of reverseMap/valuesMap – different threads are allowed to read values from them before Factory#get  for the corresponding subKey completes.

## How to run with maven (tested on v3.0.5):

#### High level test
mvn clean test -P+HighLevelTest
#### Low level test
mvn clean test -P+LowLevelTest

### How to run without maven:
```bash
tar -xvf JDKWeakCache.tgz
cd JDKWeakCache/
mkdir target
javac -d target src/test/java/weakcache/RunJavaProcessAndCaptureException.java src/test/java/weakcache/TaskExecutor.java src/test/java/weakcache/WeakCacheHighLevelTest.java src/test/java/weakcache/WeakCacheLowLevelTest.java
java -cp target weakcache.WeakCacheHighLevelTest
java -cp target weakcache.WeakCacheLowLevelTest
```

